<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="cantor_differentiate_assistant" version="1">
<MenuBar>
  <Menu name="calculate"><text>&amp;Calculate</text>
    <Action name="differentiate_assistant"/>
  </Menu>
</MenuBar>
</kpartgui>
