a.summary-letter {
	text-decoration: none
}

pre.display {
	font-family: serif
}

pre.format {
	font-family: serif
}

pre.menu-comment {
	font-family: serif
}

pre.menu-preformatted {
	font-family: serif
}

pre.smalldisplay {
	font-family: serif;
	font-size: smaller
}

pre.smallexample {
	font-size: smaller
}

pre.smallformat {
	font-family: serif;
	font-size: smaller
}

pre.smalllisp {
	font-size: smaller
}

span.sansserif {
	font-family: sans-serif;
	font-weight: normal;
}

ul.toc {
	list-style: none
}

body {
	color: black;
	background: white;
	margin-left: 8%;
	margin-right: 13%;
	font-family: "Noto Sans", sans-serif;
	line-height: 1.45;
}

h1 {
	font-size: 150%;
	font-family: "Noto Sans", sans-serif
}

h2 {
	font-size: 125%;
	font-family: "Noto Sans", sans-serif
}

h3 {
	font-size: 100%;
	font-family: "Noto Sans", sans-serif
}

a[href] {
	color: rgb(13, 13, 189, 0.75);
	text-decoration: none;
}

a[href]:hover {
	background: none;
    text-decoration: underline;
}

div.textbox {
	border: solid;
	border-width: thin;
	padding-top: 1em;
	padding-bottom: 1em;
	padding-left: 2em;
	padding-right: 2em
}

div.titlebox {
	border: none;
	padding-top: 1em;
	padding-bottom: 1em;
	padding-left: 2em;
	padding-right: 2em;
	background: rgb(200, 255, 255);
	font-family: sans-serif
}

div.synopsisbox {
	border: none;
	padding-top: 1em;
	padding-bottom: 1em;
	padding-left: 2em;
	padding-right: 2em;
	background: rgb(255, 220, 255);
}

pre.example {
	border: 2px solid rgb(190, 190, 190, 0.9);
    border-radius: 13px;
	padding-top: 1em;
	padding-bottom: 1em;
	padding-left: 1em;
	padding-right: 1em;
	background-color: rgb(220, 220, 250, 0.9)
}

div.spacerbox {
	border: none;
	padding-top: 2em;
	padding-bottom: 2em
}

div.image {
	margin: 0;
	padding: 0px;
	text-align: center
}

div.categorybox {
	border: 2px solid rgb(190, 190, 190, 0.7);
    border-radius: 13px;
	padding-top: 0px;
	padding-bottom: 0px;
	padding-left: 1em;
	padding-right: 1em;
	background-color: rgb(255, 243, 186, 0.75)
}

img {
	max-width: 80%;
	max-height: 80%
}

p {
    text-align: justify;
}

/*class section, chapter, menu
.section {

}

.chapter {

}


.menu {

}
 */
