<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="cantor_eigenvectors_assistant" version="1">
<MenuBar>
  <Menu name="linear_algebra"><text>&amp;Linear Algebra</text>
    <Action name="eigenvectors_assistant"/>
  </Menu>
</MenuBar>
</kpartgui>
